#!/usr/bin/python

# LTspice WAVE to Ftube
#
# Only for MONO, 16-BIT, WAVE
#
# Special for OneTriode by NaLex
# V1.0

import wave, struct
from pylab import *


# Tube type or prefix (only for a file name)
tube = 'Stage-1'

# Params of simulation
# Vp to define automatical
Rp = 220e3
Ri = 68e3

# Input voltage
low = -10
high = 250

# File to open
openFile = 'recto-st-1'

ext = '.wav'

# Path to file
filePath = 'Source\\'

# Plot Ftube: 0 - off, 1 - on
plotFtube = 1

# Multiply to restore
mul = 1000


def write_Ftube():
    waveFile = wave.open(filePath + openFile + ext, 'r')
    length = waveFile.getnframes()
    print ("Table size =", length)
    print ()
    tsLimit = 100e3 # table size limit
    if (length + 6) > tsLimit:
        print ("Error: Table size > %d" % tsLimit)
        return
    # create data array
    ary = [0] * length
    for i in range(0,length):
        waveData = waveFile.readframes(1)
        data = struct.unpack("<h", waveData)
        #data = struct.unpack("<b", waveData)
        data = (int(data[0]) / 32767) * mul
        data = round(data, 10)
        ary[i] = data
    # write data to file
    mVp = ary[0]
    fileName = tube + "  Vp=%dV  Rp=%dK  Ri=%dK" % (mVp, (Rp/1e3), (Ri/1e3))
    fullFilePath = filePath + fileName + '.txt'
    fileW = open(fullFilePath, 'w')
    for index in ary:
        fileW.write(str(index) + '\n')
    fileW.write(str(mVp) + '\n')
    fileW.write(str(Rp) + '\n')
    fileW.write(str(Ri) + '\n')
    fileW.write(str(low) + '\n')
    fileW.write(str(high) + '\n')
    fileW.write(str(length))
    fileW.close() # close file
    print ("File: " + fullFilePath)
    if plotFtube == 1:
        plot_Ftube(fullFilePath)

def plot_Ftube(path):
    file = open(path, 'r')
    strAry = [line.strip() for line in file]
    lenAry = len(strAry)
    if float(strAry[lenAry-1]) != (lenAry-6):
        file.close() # close file
        print ("Incorrect File")
        return
    Vi = linspace(float(strAry[lenAry-3]), float(strAry[lenAry-2]), float(strAry[lenAry-1]))
    Vp = zeros_like(Vi)
    for i, v in enumerate(Vi):
        Vp[i] = float(strAry[i])
    plot(Vi, Vp, '-') #, label="Ri")
    plot(0, 0, label="Vp=%dV" % (float(strAry[lenAry-6])))
    plot(0, 0, label="Rp=%dk" % (float(strAry[lenAry-5])/1e3))
    plot(0, 0, label="Ri=%dk" % (float(strAry[lenAry-4])/1e3))
    xlabel("Vi")
    ylabel("Vp")
    #legend(['1', '2'], loc='best')
    legend()
    #axis
    file.close() # close file
    show()

def main():
    write_Ftube()

if __name__ == "__main__":
    main()
